package com.ainur.morgoth;

import com.ainur.morgoth.net.game.GameServer;
import com.ainur.morgoth.util.MorgothArgs;
import com.ainur.morgoth.util.modules.SampleModule;
import com.google.inject.Inject;
import com.google.inject.Injector;
import com.impossibl.postgres.jdbc.PGDataSource;
import com.lexicalscope.jewel.cli.CliFactory;
import com.netflix.config.*;
import com.netflix.config.sources.JDBCConfigurationSource;
import com.netflix.governator.annotations.AutoBindSingleton;
import com.netflix.governator.configuration.ArchaiusConfigurationProvider;
import com.netflix.governator.guice.BootstrapBinder;
import com.netflix.governator.guice.BootstrapModule;
import com.netflix.governator.guice.LifecycleInjector;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.commons.configuration.SystemConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.sql.DataSource;


/**
 * Created by Jordan on 13/02/15.
 */

@AutoBindSingleton
public class Morgoth {
    private static Logger logger = LoggerFactory.getLogger(Morgoth.class);

    private Injector injector;
    private HikariDataSource dataSource;

    @Inject
    private GameServer gameServer;

    @Inject
    protected Morgoth(Injector injector) {
        this.injector = injector;
    }

    public static void main(String[] args) throws ConfigurationException {
        final MorgothArgs arguments = CliFactory.parseArguments(MorgothArgs.class, args);

        logger.info("Initialising");

        HikariDataSource hikariDataSource = initializeDataSource(arguments);
        initializeConfiguration(arguments, hikariDataSource);
        Injector injector = initializeInjector();

        Morgoth morgoth = injector.getInstance(Morgoth.class);
        morgoth.setDataSource(hikariDataSource);
        morgoth.initializeGameServer();
//        logger.info(morgoth.getDataSource().toString());
    }

    private static HikariDataSource initializeDataSource(MorgothArgs arguments) {
        logger.info("Initializing data source");
        return null;
//        HikariConfig hikariConfig = new HikariConfig();
//
//        PGDataSource dataSource = new PGDataSource();
//        dataSource.setHost(arguments.getPostgresHost());
//        dataSource.setPort(arguments.getPostgresPort());
//        dataSource.setUser(arguments.getPostgresUser());
//        dataSource.setPassword(arguments.getPostgresPassword());
//        dataSource.setDatabase(arguments.getPostgresDatabase());
//
//        hikariConfig.setDataSource(dataSource);
//        hikariConfig.setUsername(arguments.getPostgresUser());
//        hikariConfig.setPassword(arguments.getPostgresPassword());
//
//        return new HikariDataSource(hikariConfig);
    }

    private static Injector initializeInjector() {
        logger.info("Creating lifecycle injector");
        return LifecycleInjector
                .builder()
                .usingBasePackages("com.ainur.morgoth")
                .withBootstrapModule(
                        new BootstrapModule() {
                            @Override
                            public void configure(BootstrapBinder binder) {
                                binder.bindConfigurationProvider().toInstance(ArchaiusConfigurationProvider.builder().build());
                            }
                        }
                )
                .withModuleClass(SampleModule.class)
                .build().createInjector();
    }

    private static void initializeConfiguration(MorgothArgs arguments, DataSource dataSource) throws ConfigurationException {
        logger.info("Loading configuration from {}, environment and database", arguments.getConfigurationBootstrap());
        ConcurrentMapConfiguration propertiesConfig = new ConcurrentMapConfiguration((new PropertiesConfiguration(arguments.getConfigurationBootstrap())));
        ConcurrentMapConfiguration systemConfig = new ConcurrentMapConfiguration(new SystemConfiguration());
//
//        PolledConfigurationSource polledConfigurationSource = new JDBCConfigurationSource(
//                dataSource,
//                "Select distinct property_key, property_value from server_properties",
//                "property_key",
//                "property_value"
//        );
//
//        DynamicConfiguration dynamicConfiguration = new DynamicConfiguration(polledConfigurationSource,
//                new FixedDelayPollingScheduler(100, 1000, true));

        ConcurrentCompositeConfiguration finalConfig = new ConcurrentCompositeConfiguration();

        finalConfig.addConfiguration(propertiesConfig, "propertiesConfig");
        finalConfig.addConfiguration(systemConfig, "systemConfig");
//        finalConfig.addConfiguration(dynamicConfiguration, "dynamicConfig");

        ConfigurationManager.install(finalConfig);
    }

    private void initializeGameServer() {
        gameServer.startServer();
    }

    public Injector getInjector() {
        return injector;
    }

    public HikariDataSource getDataSource() {
        return dataSource;
    }

    protected void setDataSource(HikariDataSource dataSource) {
        this.dataSource = dataSource;
    }
}
