package com.ainur.morgoth.net.game;

import com.ainur.morgoth.net.game.protocol.ClientMessage;
import com.ainur.morgoth.net.game.sessions.GameSession;
import com.ainur.morgoth.net.game.sessions.GameSessionManager;
import com.google.inject.Inject;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Handles various events related to the current connection.
 */
public class GameHandler extends SimpleChannelInboundHandler<ClientMessage> {
    private static final Logger logger = LoggerFactory.getLogger(GameHandler.class);
    private GameSession session;

    @Inject
    private GameServer gameServer;

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        GameServer.getChannelGroup().add(ctx.channel());
        this.session = GameServer.getSessionManager().addConnection(ctx.channel());
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) {
        GameServer.getChannelGroup().remove(ctx.channel());
        GameServer.getSessionManager().removeConnection(ctx.channel().attr(GameSessionManager.SESSION_ID_ATTR).get());
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable e) {
        logger.error("Caught exception", e);
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, ClientMessage msg) throws Exception {
        logger.info("Received message [Id: " + msg.opCode + "] Data: " + msg.toString());
    }
}