package com.ainur.morgoth.net.game;

import com.ainur.morgoth.net.game.codec.GameDecoder;
import com.ainur.morgoth.net.game.codec.GameEncoder;
import com.ainur.morgoth.net.game.codec.PolicyDecoder;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.util.CharsetUtil;

/**
 * Created by Jordan on 26/03/15.
 */
public class GamePipelineFactory extends ChannelInitializer<SocketChannel> {
    private final String policyFile;
    private EventLoopGroup channelHandlerGroup;

    public GamePipelineFactory(EventLoopGroup channelHandlerGroup, String policyFile) {
        this.channelHandlerGroup = channelHandlerGroup;
        this.policyFile = policyFile;
    }

    @Override
    protected void initChannel(SocketChannel ch) throws Exception {
        final ChannelPipeline channelPipeline = ch.pipeline();

        channelPipeline.addLast("policy", new PolicyDecoder(policyFile));
        channelPipeline.addLast("stringEncoder", new StringEncoder(CharsetUtil.UTF_8));
        channelPipeline.addLast("decoder", new GameDecoder());
        channelPipeline.addLast("encoder", new GameEncoder());
        channelPipeline.addLast("handler", new GameHandler());
        channelPipeline.addLast(this.channelHandlerGroup, "pipelineExecutor", new GameHandler());
    }
}
