package com.ainur.morgoth.net.game;

import com.ainur.morgoth.net.game.sessions.GameSessionManager;
import com.netflix.governator.annotations.Configuration;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.buffer.UnpooledByteBufAllocator;
import io.netty.channel.ChannelException;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoop;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.epoll.Epoll;
import io.netty.channel.epoll.EpollEventLoopGroup;
import io.netty.channel.epoll.EpollServerSocketChannel;
import io.netty.channel.group.ChannelGroup;
import io.netty.channel.group.DefaultChannelGroup;
import io.netty.channel.nio.NioEventLoopGroup;

import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.util.concurrent.GlobalEventExecutor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.InetSocketAddress;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

/**
 * Created by Jordan on 26/03/15.
 * Upgraded to Netty 4 by Leon ;-)
 */
public class GameServer {
    private static final Logger logger = LoggerFactory.getLogger(GameServer.class);
    private static ChannelGroup channelGroup;
    private static GameSessionManager gameSessionManager;

    @Configuration(value = "com.ainur.morgoth.net.game.GameServer.port")
    private int port;

    @Configuration(value = "com.ainur.morgoth.net.game.GameServer.host")
    private String host;

    @Configuration(value = "com.ainur.morgoth.util.FlashPolicy.file")
    private String policyFile;

    @Configuration(value = "com.ainur.morgoth.net.game.GameServer.eventLoopSize")
    private int eventLoopSize = 8;

    @Configuration(value = "com.ainur.morgoth.net.game.GameServer.preferEpoll")
    private boolean preferEpoll = false;

    private static ServerBootstrap serverBootstrap;

    public static ChannelGroup getChannelGroup() {
        return channelGroup;
    }

    public void startServer() {
        boolean epoll = preferEpoll && Epoll.isAvailable();

        final EventLoopGroup acceptGroup = epoll ? new EpollEventLoopGroup(this.eventLoopSize) : new NioEventLoopGroup(this.eventLoopSize);
        final EventLoopGroup ioGroup = epoll ? new EpollEventLoopGroup(this.eventLoopSize) : new NioEventLoopGroup(this.eventLoopSize);
        final EventLoopGroup channelHandlerGroup = epoll ? new EpollEventLoopGroup(this.eventLoopSize) : new NioEventLoopGroup(this.eventLoopSize);

        serverBootstrap = new ServerBootstrap()
                .group(acceptGroup, ioGroup)
                .channel(epoll ? EpollServerSocketChannel.class : NioServerSocketChannel.class)
                .childHandler(new GamePipelineFactory(channelHandlerGroup, policyFile))
                .option(ChannelOption.TCP_NODELAY, true)
                .option(ChannelOption.SO_KEEPALIVE, true)
                .childOption(ChannelOption.TCP_NODELAY, true)
                .childOption(ChannelOption.SO_KEEPALIVE, true)
                .childOption(ChannelOption.ALLOCATOR, UnpooledByteBufAllocator.DEFAULT);

        channelGroup = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);
        gameSessionManager = new GameSessionManager();

        try {
            // Not sure why you're doing it like this; if it doesn't work properly, do it like this:

//            bootstrap.bind(new InetSocketAddress(ip, port)).addListener(objectFuture -> {
//                if(!objectFuture.isSuccess()) {
//                      logger.error("Failed to bind server.");
//                }
//            });

            channelGroup.add(serverBootstrap.bind(new InetSocketAddress(host, port)).channel());
        } catch (ChannelException e) {
            logger.error("Failed to bind server.", e);
        }

        logger.info("Server bound on {}:{}", host, port);
    }

    public static GameSessionManager getSessionManager() {
        return gameSessionManager;
    }
}
